/**
 *
 * Copyright 2003-2004 The Apache Software Foundation
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */

package org.apache.geronimo.kernel;

import javax.management.InstanceAlreadyExistsException;
import javax.management.InstanceNotFoundException;
import javax.management.MBeanServer;
import javax.management.ObjectName;

import org.apache.geronimo.gbean.jmx.GBeanMBean;
import org.apache.geronimo.kernel.config.ConfigurationManager;
import org.apache.geronimo.kernel.config.InvalidConfigException;

/**
 *
 *
 * @version $Revision: 1.7 $ $Date: 2004/03/10 09:59:01 $
 */
public interface KernelMBean {
    /**
     * Get the MBeanServer used by this kernel
     * @return the MBeanServer used by this kernel
     */
    MBeanServer getMBeanServer();

    /**
     * Get the name of this kernel
     * @return the name of this kernel
     */
    String getKernelName();

    /**
     * Load a specific GBean into this kernel.
     * This is intended for applications that are embedding the kernel.
     * @param name the name to register the GBean under
     * @param gbean the GBean to register
     * @throws InstanceAlreadyExistsException if the name is already used
     * @throws org.apache.geronimo.kernel.config.InvalidConfigException if there is a problem during registration
     */
    void loadGBean(ObjectName name, GBeanMBean gbean) throws InstanceAlreadyExistsException, InvalidConfigException;

    /**
     * Start a specific GBean.
     * @param name the GBean to start
     * @throws InstanceNotFoundException if the GBean could not be found
     */
    void startGBean(ObjectName name) throws InstanceNotFoundException, InvalidConfigException;

    /**
     * Start a specific GBean and its children.
     * @param name the GBean to start
     * @throws javax.management.InstanceNotFoundException if the GBean could not be found
     */
    void startRecursiveGBean(ObjectName name) throws InstanceNotFoundException, InvalidConfigException;

    /**
     * Stop a specific GBean.
     * @param name the GBean to stop
     * @throws javax.management.InstanceNotFoundException if the GBean could not be found
     */
    void stopGBean(ObjectName name) throws InstanceNotFoundException, InvalidConfigException;

    /**
     * Unload a specific GBean.
     * This is intended for applications that are embedding the kernel.
     * @param name the name of the GBean to unregister
     * @throws javax.management.InstanceNotFoundException if the GBean could not be found
     */
    void unloadGBean(ObjectName name) throws InstanceNotFoundException;

    boolean isRunning();

    ConfigurationManager getConfigurationManager();

    Object getAttribute(ObjectName objectName, String attributeName) throws Exception;

    void setAttribute(ObjectName objectName, String attributeName, Object attributeValue) throws Exception;

    Object invoke(ObjectName objectName, String methodName) throws Exception;

    Object invoke(ObjectName objectName, String methodName, Object[] args, String[] types) throws Exception;

    boolean isLoaded(ObjectName name);
}
